
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>

#include "./vpdetection/updator.h"
#include "./vpdetection/VPCluster.h"
#include "./vpdetection/VPSample.h"

#include "mfg.h"
#include "mfg_utils.h"

using namespace std;


// exe infilename outputfilename
vector<cv::Mat> detectVP_Jlink(vector<LineSegmt2d>& lines, std::vector<unsigned int>& LableCount, double lenThresh)
{
	vector<int> hIdx;
	vector<LineSegmt2d> hlines;
	for(int i=0; i < lines.size(); ++i) {
		if (lines[i].vpLid < 0 && lines[i].length() > lenThresh) {
			hIdx.push_back(i);
			hlines.push_back(lines[i]);
		}
	}

	std::vector< std::vector<float> *> pts;
	for(int i=0; i < hlines.size(); ++i) {
		vector<float>* ln = new std::vector<float>(4);
		pts.push_back(ln);
		(*ln)[0]=(float)hlines[i].endpt1.x;
		(*ln)[1]=(float)hlines[i].endpt1.y;
		(*ln)[2]=(float)hlines[i].endpt2.x;
		(*ln)[3]=(float)hlines[i].endpt2.y;
	}
	std::vector<unsigned int> Lables;
	
	//2. VP cluster
		std::vector<std::vector<float> *> *mModels = 
			VPSample::run(&pts, 1000, 2, 0, 3);
		int classNum = VPCluster::run(Lables, LableCount, &pts, mModels, 2, 2);
	
		vector<vector<int>> groups(classNum);
		for(int i=0; i < hlines.size(); ++i) {
			lines[hIdx[i]].vpLid = Lables[i] + 1;
			groups[Lables[i]].push_back(i);
		}

		//2.1. release other resource
		for(unsigned int i=0; i < mModels->size(); ++i)
			delete (*mModels)[i];
		delete mModels;
	
	for(unsigned int i=0; i < pts.size(); i++)
		delete pts[i];

	vector<cv::Mat> vps;
	for(int i=0; i < classNum; ++i) {
		vector<LineSegmt2d> group(groups[i].size());
		for (int j=0; j < groups[i].size(); ++j) 
			group[j] = hlines[groups[i][j]];
		cv::Mat vp;
		cv::Mat A(3,group.size(), CV_64F); // A'*vp = 0;
		for (int i = 0; i < group.size(); ++i) {
			cv::Mat eq = group[i].lineEq();
			eq.copyTo(A.col(i));
		}
		if (A.cols==2) {
			cv::SVD svd(A.t());	
			vp = svd.vt.row(2).t();
		} else 
			cv::SVD::solveZ(A.t(),vp);
	
	// compute optimal vp with given line segments
		if (group.size() >2) {
			optimizeVainisingPoint (group, vp);
		}
		vps.push_back(vp);

	}

	return vps;
}

vector<cv::Mat> detectVP_Jlink(vector<LineSegmt2d>& lines, std::vector<unsigned int>& LableCount, double lenThresh,
	vector<cv::Mat>& vpCov)
{
	vector<int> hIdx;
	vector<LineSegmt2d> hlines;
	for(int i=0; i < lines.size(); ++i) {
		if (lines[i].vpLid < 0 && lines[i].length() > lenThresh) {
			hIdx.push_back(i);
			hlines.push_back(lines[i]);
		}
	}

	std::vector< std::vector<float> *> pts;
	for(int i=0; i < hlines.size(); ++i) {
		vector<float>* ln = new std::vector<float>(4);
		pts.push_back(ln);
		(*ln)[0]=(float)hlines[i].endpt1.x;
		(*ln)[1]=(float)hlines[i].endpt1.y;
		(*ln)[2]=(float)hlines[i].endpt2.x;
		(*ln)[3]=(float)hlines[i].endpt2.y;
	}
	std::vector<unsigned int> Lables;
	
	//2. VP cluster
		std::vector<std::vector<float> *> *mModels = 
			VPSample::run(&pts, 1000, 2, 0, 3);
		int classNum = VPCluster::run(Lables, LableCount, &pts, mModels, 2, 2);
	
		vector<vector<int>> groups(classNum);
		for(int i=0; i < hlines.size(); ++i) {
			lines[hIdx[i]].vpLid = Lables[i] + 1;
			groups[Lables[i]].push_back(i);
		}

		//2.1. release other resource
		for(unsigned int i=0; i < mModels->size(); ++i)
			delete (*mModels)[i];
		delete mModels;
	
	for(unsigned int i=0; i < pts.size(); i++)
		delete pts[i];

	vector<cv::Mat> vps;
	for(int i=0; i < classNum; ++i) {
		vector<LineSegmt2d> group(groups[i].size());
		for (int j=0; j < groups[i].size(); ++j) 
			group[j] = hlines[groups[i][j]];
		cv::Mat vp;
		cv::Mat A(3,group.size(), CV_64F); // A'*vp = 0;
		for (int i = 0; i < group.size(); ++i) {
			cv::Mat eq = group[i].lineEq();
			eq.copyTo(A.col(i));
		}
		if (A.cols==2) {
			cv::SVD svd(A.t());	
			vp = svd.vt.row(2).t();
		} else 
			cv::SVD::solveZ(A.t(),vp);
	
	// compute optimal vp with given line segments
		cv::Mat cov, cov_tmp;
		if (group.size() >2) {
			optimizeVainisingPoint (group, vp, cov, cov_tmp);
		}
		vps.push_back(vp);
		vpCov.push_back(cov);
	}

	return vps;
}
