#include "mfg.h"
#include "mfg_utils.h"

struct Data_EST3D
{
	vector<cv::Mat> Rs,  ts;
	vector<cv::Point2d>  pt;
	cv::Mat K;
};

void costFun_EST3D(double *p, double *error, int numPara, int numMeas, void *adata)
{
	
	struct Data_EST3D* dp = (struct Data_EST3D *) adata;
	cv::Mat X = (cv::Mat_<double>(3,1)<<p[0], p[1], p[2]);
	double cost=0;

	for(int i=0; i < dp->pt.size(); ++i) {
		cv::Point2d pi = mat2cvpt(dp->K *(dp->Rs[i] * X + dp->ts[i]));
		error[i*2] = pi.x - dp->pt[i].x;
		error[i*2+1] = pi.y - dp->pt[i].y;

		cost = cost + error[i*2]*error[i*2] + error[i*2+1]*error[i*2+1];
	}

}

void est3dpt (vector<cv::Mat> Rs, vector<cv::Mat> ts, cv::Mat K, vector<cv::Point2d> pt, cv::Mat& X, int maxIter)
// input: Rs, ts, pt
// output: X
{
	// ----- LM parameter setting -----
	double opts[LM_OPTS_SZ], info[LM_INFO_SZ];
	opts[0] = LM_INIT_MU; //
	opts[1] = 1E-50; // gradient threshold, original 1e-15
	opts[2] = 1E-20; // relative para change threshold? original 1e-50
	opts[3] = 1E-20; // error threshold (below it, stop)
	opts[4] = LM_DIFF_DELTA;


	int numPara = 3, numMeas = pt.size()*2;
	double* para = new double[numPara];
	para[0] = X.at<double>(0);
	para[1] = X.at<double>(1);
	para[2] = X.at<double>(2);
	double* meas = new double[numMeas];
	for(int i=0; i<numMeas; ++i) meas[i] = 0;

	Data_EST3D data;
	data.Rs = Rs;
	data.ts = ts;
	data.pt = pt;
	data.K = K;

	// ----- start LM solver -----
	int ret = dlevmar_dif(costFun_EST3D, para, meas, numPara, numMeas,
						maxIter, opts, info, NULL, NULL, (void*)&data);
	delete[] meas;	

	X = (cv::Mat_<double>(3,1)<<para[0], para[1], para[2]);
}

cv::Mat triangulatePoint_nonlin (const cv::Mat& R1, const cv::Mat& t1, const cv::Mat& R2, 
			const cv::Mat& t2,const cv::Mat& K,	cv::Point2d pt1, cv::Point2d pt2)
// nonlinear triangulation of 3d pt
{
	vector<cv::Mat> Rs, ts;
	Rs.push_back(R1);
	Rs.push_back(R2);
	ts.push_back(t1);
	ts.push_back(t2);
	vector<cv::Point2d> pts;
	pts.push_back(pt1);
	pts.push_back(pt2);
	cv::Mat X = triangulatePoint (R1, t1, R2, t2, K, pt1, pt2); // linear triangulation
	est3dpt (Rs, ts, K, pts, X, 80);

	return X;
}

// ***********************************************************************************
//************************************************************************************

struct Data_SCALE
{
	cv::Mat Rn, Rtn_1, t;
	vector<cv::Point3d>  X;
	vector<cv::Point2d>  x;
	cv::Mat K;
};

void costFun_Scale(double *p, double *error, int numPara, int numMeas, void *adata)
{
	struct Data_SCALE* dp = (struct Data_SCALE *) adata;

	cv::Mat tn = dp->Rtn_1 + p[0] * dp->t  ;
	double cost = 0;
	for(int i=0; i< dp->X.size(); ++i) {
		cv::Point2d xi = mat2cvpt(dp->K * (dp->Rn * cvpt2mat(dp->X[i],0) + tn));
		error[i*2] = xi.x - dp->x[i].x;
		error[i*2+1] = xi.y - dp->x[i].y;

		cost += error[i*2]*error[i*2] + error[i*2+1]*error[i*2+1];
	}
	cout<<cost<<"\t";
}

double optimizeScale (vector<cv::Point3d> X, vector<cv::Point2d> x, cv::Mat K, 
					cv::Mat Rn, cv::Mat Rtn_1, cv::Mat t, double s)
{
	cv::Mat tn = Rtn_1 + t * s;
	
	// ----- LM parameter setting -----
	double opts[LM_OPTS_SZ], info[LM_INFO_SZ];
	opts[0] = LM_INIT_MU; //
	opts[1] = 1E-50; // gradient threshold, original 1e-15
	opts[2] = 1E-200; // relative para change threshold? original 1e-50
	opts[3] = 1E-20; // error threshold (below it, stop)
	opts[4] = LM_DIFF_DELTA;
	int maxIter = 500;


	int numPara = 1, numMeas = x.size() * 2;
	double* para = new double[numPara];
	para[0] = s;
	double* meas = new double[numMeas];
	for(int i=0; i < numMeas; ++i) meas[i] = 0;

	Data_SCALE data;
	data.Rn = Rn;
	data.Rtn_1 = Rtn_1;
	data.t = t;
	data.X = X;
	data.x = x;
	data.K = K;

	// ----- start LM solver -----
	int ret = dlevmar_dif(costFun_Scale, para, meas, numPara, numMeas,
						maxIter, opts, info, NULL, NULL, (void*)&data);
	delete[] meas;	
//	termReason((int)info[6]);
	
	return para[0];
}