/* ---------------------------------------------------------------------------
MFG - Multilayer Feature Graph for Robot Navigation

This is the main file for C++ Implementation of MFG
Author: Yan Lu 
@ NetBot Lab, TAMU (http://telerobot.cs.tamu.edu/) 
-----------------------------------------------------------------------------*/

//------------------------------ Header files ---------------------------------

#include <QApplication>
#include <QDesktopWidget>
#include <math.h>
#include "window.h"
#include "mfg.h"
#include "mfg_utils.h"

using namespace std; 

//------------------------------ Global variables -----------------------------
int IDEAL_IMAGE_WIDTH;
double THRESH_POINT_MATCH_RATIO = 0.50;

double SIFT_THRESH_HIGH = 0.05;
double SIFT_THRESH_LOW  = 0.03; // 

double SIFT_THRESH = SIFT_THRESH_HIGH;

SysPara syspara;
vector<vector<double>> planeColors;

void SysPara::init()
{
	use_img_width = -1;
	kpt_detect_alg = 1; 

	gftt_max_ptnum = 300; 
	gftt_qual_levl = 0.01;
	gftt_min_ptdist = 5; 

	oflk_min_eigval = 0.001;
	oflk_win_size = 19; 

	ini_increment = 80;
	frm_increment = 4; 
	kpt_desc_radius = 21; 
	 
	nFrm4VptBA = 10;

	mfg_pt2plane_dist = 0.16;
	mfg_num_recent_pts = 400; 
	mfg_num_recent_lns = 50; 
	mfg_min_npt_plane = 100; 

	ba_weight_vp = 15;
	ba_weight_ln = 1;
	ba_weight_pl = 100;

	ba_use_kernel = true;
	ba_kernel_delta_pt = 1;
	ba_kernel_delta_vp = 1;
	ba_kernel_delta_ln = 3;
	ba_kernel_delta_pl = 1;

	angle_thresh_vp2d3d = 5; // degree

	updatingMFG = false;

	cpu_trackpt = 0; cpu_LBA = 0; 
	cpu_3dpt = 0; cpu_3dln = 0; cpu_3dvp = 0; cpu_3dpl = 0;
	cpu_estmot = 0;
	cpu_view3 = 0; cpu_view4 = 0; cpu_kpdet = 0; cpu_lsdet = 0; cpu_ildet = 0; cpu_vpdet = 0;
}

//------------------------------- Main function -------------------------------
int main(int argc, char *argv[])
{		
	// -----------  Qt GUI ------------ 
	QApplication app(argc, argv);
	Window win;
	win.resize(win.sizeHint());
	
	syspara.init();	
	for (int i=0; i<100; ++i) {
		vector<double> color(3);
		color[0] = (rand()%100)/100.0;
		color[1] = (rand()%100)/100.0;
		color[2] = (rand()%100)/100.0;
		planeColors.push_back(color);
	}

	srand(1); 
	string imgName;			// first image name
	cv::Mat K, distCoeffs;	// distortion coeff: k1, k2, p1, p2
	int imIdLen = 5;			// n is the image number length,
	int ini_incrt = syspara.ini_increment;
	int increment = syspara.frm_increment;
	int totalImg = 12500;
	getConfigration2 (&imgName, K, distCoeffs, syspara.use_img_width) ;
	
	//------- initialize -------
	View view0(imgName, K, distCoeffs, 0);
	view0.frameId = atoi (imgName.substr(imgName.size()-imIdLen-4, imIdLen).c_str());
	imgName = nextImgName(imgName, imIdLen, ini_incrt);
	View view1(imgName, K, distCoeffs, 1);
	view1.frameId = atoi (imgName.substr(imgName.size()-imIdLen-4, imIdLen).c_str());

	Mfg map(view0, view1, 30);	
	
	syspara.kpt_detect_alg = 3; 

	MfgThread mthread;
	mthread.pMap = &map;
	mthread.imgName = imgName;
	mthread.increment = increment;
	mthread.totalImg = totalImg;
	mthread.imIdLen = imIdLen;
	mthread.K = K;
	mthread.distCoeffs = distCoeffs;

	mthread.start();
	// ------- plot ---------
	win.setMfgScene(&map);
	int desktopArea = QApplication::desktop()->width() *
		QApplication::desktop()->height();
	int widgetArea = win.width() * win.height();
	if (((float)widgetArea / (float)desktopArea) < 0.75f)
		win.show();
	else
		win.showMaximized();
	return app.exec();

}

